/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/model/model.hpp>

#include <nostalgia/foundation/asset.hpp>
#include <nostalgia/foundation/module.hpp>

#include "gfx.hpp"
#include "typeconv.hpp"

#include "module.hpp"

namespace nostalgia::core {

class CoreModule: public foundation::Module {
	private:
		NostalgiaPaletteToPaletteConverter nostalgiaPaletteToPaletteConverter;
		TileSheetV1ToTileSheetConverter nostalgiaGraphicToTileSheetConverter;
		TileSheetToCompactTileSheetConverter tileSheetToCompactTileSheetConverter;
		TileSheetV2ToTileSheetConverter tileSheetV2ToTileSheetConverter;

	public:
		static CoreModule mod;
		[[nodiscard]]
		ox::Vector<foundation::TypeDescGenerator> types() const noexcept override;
		[[nodiscard]]
		ox::Vector<const foundation::BaseConverter*> converters() const noexcept override;
		[[nodiscard]]
		ox::Vector<foundation::PackTransform> packTransforms() const noexcept override;
};

CoreModule CoreModule::mod;

const foundation::Module *module() noexcept {
	return &CoreModule::mod;
}

ox::Vector<foundation::TypeDescGenerator> CoreModule::types() const noexcept {
	return {
		foundation::generateTypeDesc<TileSheetV1>,
		foundation::generateTypeDesc<TileSheetV2>,
		foundation::generateTypeDesc<TileSheet>,
		foundation::generateTypeDesc<CompactTileSheet>,
		foundation::generateTypeDesc<NostalgiaPalette>,
		foundation::generateTypeDesc<Palette>,
	};
}

ox::Vector<const foundation::BaseConverter*> CoreModule::converters() const noexcept {
	return {
		&nostalgiaPaletteToPaletteConverter,
		&nostalgiaGraphicToTileSheetConverter,
		&tileSheetToCompactTileSheetConverter,
		&tileSheetV2ToTileSheetConverter,
	};
}

ox::Vector<foundation::PackTransform> CoreModule::packTransforms() const noexcept {
	return {
		// convert tilesheets to CompactTileSheets
		[](foundation::Context *ctx, ox::Buffer *buff) -> ox::Error {
			oxRequire(hdr, foundation::readAssetHeader(*buff));
			const auto typeId = ox::buildTypeId(hdr.clawHdr.typeName, hdr.clawHdr.typeVersion, hdr.clawHdr.typeParams);
			if (typeId == ox::buildTypeId<TileSheetV1>() ||
			    typeId == ox::buildTypeId<TileSheetV2>() ||
			    typeId == ox::buildTypeId<TileSheet>()) {
				oxReturnError(foundation::convertBuffToBuff<core::CompactTileSheet>(
							ctx, *buff, ox::ClawFormat::Metal).moveTo(buff));
			}
			return {};
		},
	};
}

}
