/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <nostalgia/core/core.hpp>
#include <nostalgia/scene/scene.hpp>

using namespace nostalgia;

static bool s_paused = false;
static ox::Optional<scene::Scene> s_scene;

static int updateHandler(core::Context*) noexcept {
	constexpr auto sleepTime = 16;
	if (s_paused) {
		return sleepTime;
	}
	// do stuff
	return sleepTime;
}

static void keyEventHandler(core::Context *ctx, core::Key key, bool down) noexcept {
	if (down) {
		if (key == core::Key::Alpha_Q) {
			core::shutdown(ctx);
		} else if (key == core::Key::Alpha_P) {
			s_paused = !s_paused;
		}
	}
}

ox::Error run(ox::UniquePtr<ox::FileSystem> fs) noexcept {
	oxTraceInitHook();
	oxRequireM(ctx, core::init(std::move(fs)));
	constexpr ox::FileAddress SceneAddr("/Scenes/Chester.nscn");
	oxRequire(scn, foundation::readObj<scene::SceneStatic>(ctx.get(), SceneAddr));
	core::setUpdateHandler(ctx.get(), updateHandler);
	core::setKeyEventHandler(ctx.get(), keyEventHandler);
	s_scene.emplace(*scn);
	oxReturnError(s_scene->setupDisplay(ctx.get()));
	return core::run(ctx.get());
}
